const moment = require('/miniprogram_npm/moment/index.js');
Component({
  properties: {
    activityTimeList: {
      type: Array,
      value: [],
      observer: 'getCommentList'
    },
  },
  data: {
    year: 0,
    month: 0,
    calendarDays: [],
    reminders: {},
    showReminderModal: false,
    showAddReminderModal: false,
    selectedDate: '',
    reminderInput: '',
  },
  methods: {
    getCommentList() {
      const currentDate = new Date();
      this.setData({
        year: currentDate.getFullYear(),
        month: currentDate.getMonth() + 1,
      });

      this.generateCalendar(this.data.year, this.data.month);
    },
    // 生成指定年月的日历数据
    generateCalendar(year, month) {
      const firstDay = new Date(year, month - 1, 1);
      const firstDayOfWeek = firstDay.getDay();
      const lastDay = new Date(year, month, 0);
      const daysInMonth = lastDay.getDate();

      const days = [];
      // 填充月初的空白
      for (let i = 0; i < firstDayOfWeek; i++) {
        days.push({
          day: null,
          date: null,
          hasReminder: false
        });
      }
      // 填充日期
      for (let day = 1; day <= daysInMonth; day++) {
        const date = `${year}-${month.toString().padStart(2, '0')}-${day.toString().padStart(2, '0')}`;
        const hasReminder = this.properties.activityTimeList.some(e => e === date)
        days.push({
          day,
          date,
          hasReminder
        });
      }
      this.setData({
        calendarDays: days
      });
    },

    // 切换到上一月
    handlePrevMonth() {
      let {
        year,
        month
      } = this.data;
      if (month === 1) {
        year--;
        month = 12;
      } else {
        month--;
      }
      this.setData({
        year,
        month
      });
      this.generateCalendar(year, month);
    },

    // 切换到下一月
    handleNextMonth() {
      let {
        year,
        month
      } = this.data;
      if (month === 12) {
        year++;
        month = 1;
      } else {
        month++;
      }
      this.setData({
        year,
        month
      });
      this.generateCalendar(year, month);
    },

    // 处理日期点击事件
    handleDayClick(e) {
      const date = e.currentTarget.dataset.date;
      this.triggerEvent('myevent', date + ' 00:00:00')
      // if (date) {
      //   this.setData({
      //     selectedDate: date
      //   });
      //   if (this.data.reminders[date] && this.data.reminders[date].length > 0) {
      //     this.setData({
      //       showReminderModal: true
      //     });
      //   } else {
      //     this.setData({
      //       showAddReminderModal: true
      //     });
      //   }
      // }
    },

    // 关闭提醒详情模态框
    closeReminderModal() {
      this.setData({
        showReminderModal: false
      });
    },

    // 处理提醒输入
    handleReminderInput(e) {
      this.setData({
        reminderInput: e.detail.value
      });
    },

    // 添加提醒
    addReminder() {
      const {
        selectedDate,
        reminderInput,
        reminders
      } = this.data;
      if (reminderInput) {
        if (!reminders[selectedDate]) {
          reminders[selectedDate] = [];
        }
        reminders[selectedDate].push(reminderInput);
        this.setData({
          reminders,
          showAddReminderModal: false,
          reminderInput: ''
        });
        this.generateCalendar(this.data.year, this.data.month);
      }
    },

    // 取消添加提醒
    cancelAddReminder() {
      this.setData({
        showAddReminderModal: false,
        reminderInput: ''
      });
    }
  }
});