const {
  get,
  post
} = require('../../../utils/request');
import url from '../../../utils/urlconfig'
Component({
  properties: {
    // 这里定义了newsId属性，属性值可以在组件使用时指定
    newsId: {
      type: String,
      value: '',
      observer: 'getCommentList'
    },
    favoriteState: {
      type: String,
      value: '',
      observer: "changefavoriteStu"
    },
    uploadImgStu: {
      type: Boolean,
      value: false
    },
    favoriteShow: {
      type: Boolean,
      value: false
    },
    bizType: {
      type: String,
      value: 2
    }
  },
  data: {
    // 这里是一些组件内部数据
    commentList: [],
    imageList: [],
    commentReplyList: [],
    showModal: false,
    showMoreStatu: false,
    favoriteStu: false,
    replyVal: "",
    commentVal: "",
    tpCommentId: "",
    commentTotal: "",
    uploadImgs:[]
  },
  lifetimes: {
    attached: function () {

    },
    ready: function () {

    },
    detached: function () {
      // 在组件实例被从页面节点树移除时执行
    },
  },

  methods: {
    // 这里是一个自定义方法
    getCommentList: function () {
      get(url.getCommentListApi, {
        id: this.properties.newsId,
        bizType: this.properties.bizType == 1 ? '2' : '',
        pageNo: 1,
        pageSize: 30,
      }).then(res => {
        let list = []
        let resultList = []
        res.data.records.forEach(item => {
          if (item.tpCommentReplies.length > 0) {
            resultList = list.concat(item.tpCommentReplies)
          }
          if(item.imageUrls){
            item.imageUrls = JSON.parse(item.imageUrls);
          }
          
        });
        this.setData({
          commentList: res.data.records,
          commentTotal: res.data.total,
          commentReplyList: resultList
        })
      })
    },

    //点赞
    laudClick: function (e) {
      let item = e.currentTarget.dataset.item
      let params = {
        tpCommentId: item.id,
      };
      if (String(item.status) === "0") {
        post(url.trainReviewGoodApi, params).then(res => {
          wx.showToast({
            title: '点赞成功',
            icon: 'success',
            duration: 2000
          })
          this.getCommentList()
        })
      } else {
        post(url.trainReviewBadApi, params).then(res => {
          wx.showToast({
            title: '取消点赞',
            icon: 'success',
            duration: 2000
          })
          this.getCommentList()
        })
      }
    },
    //回复评论input
    replyValBindinput: function (e) {
      this.setData({
        replyVal: e.detail.value
      })
    },
    //评论input
    commentValBindinput: function (e) {
      this.setData({
        commentVal: e.detail.value
      })
    },
    // 展开更多回复
    showMore: function () {
      this.setData({
        showMoreStatu: true
      })
    },
    //回复
    reviewDetail: function (e) {
      this.setData({
        showModal: true,
        tpCommentId: e.currentTarget.dataset.item.id
      })
    },
    //改变收藏状态
    /**
     * favoriteState 1：已收藏 0：未收藏
     */
    changefavoriteStu() {
      if (this.data.favoriteState == '1') {
        this.setData({
          favoriteStu: true
        })
      } else {
        this.setData({
          favoriteStu: false
        })
      }
    },
    // 收藏
    setCollection() {
      if (this.data.favoriteStu) {
        get(url.setCancelFavoriteApi, {
          studentCaseId: this.data.newsId
        }).then(res => {
          wx.showToast({
            title: '取消收藏',
            icon: 'success',
            duration: 2000,
            success: () => {
              this.setData({
                favoriteStu: false
              })
            }
          })
        })
      } else {
        get(url.setFavoriteApi, {
          studentCaseId: this.data.newsId
        }).then(res => {
          wx.showToast({
            title: '收藏成功',
            icon: 'success',
            duration: 2000,
            success: () => {
              this.setData({
                favoriteStu: true
              })
            }
          })
        })
      }
    },
    //相册调用
    uploadImg() {
      wx.chooseImage({
        count: 4,
        sizeType: ['original', 'compressed'],
        sourceType: ['album', 'camera'],
        success: (res) => {
          // tempFilePath可以作为img标签的src属性显示图片
          const tempFilePaths = res.tempFilePaths;
          if (this.data.imageList.length > 3) {
            wx.showToast({
              title: '最多上传四张',
              icon: 'error',
              duration: 2000,
            })
          } else {
            const TOKEN_KEY = 'auth_token';
            const tokenData = wx.getStorageSync(TOKEN_KEY);
            wx.uploadFile({
              url: 'https://hqjddj.shanghaiairport.com/aliyun/api/upload/?companyCode=zhdj&siteCode=edu', // 服务器接口地址
              filePath: tempFilePaths[0], // 单张图片路径
              name: 'file', // 后台接收的字段名
              header: {
                'content-type': 'application/json', // 默认JSON格式
                'Authorization': tokenData.token,
              },
              success: (res) => {
                const data = JSON.parse(res.data); // 解析服务器响应
                this.data.uploadImgs.push(data.data)
                this.setData({
                  imageList: [...this.data.imageList, ...tempFilePaths]
                });
              },
            });

          }

        }
      })
    },
    //图片删除
    deleteImage(e) {
      const index = e.currentTarget.dataset.index;
      const imageList = this.data.imageList.filter((_, i) => i !== index);
      this.setData({
        imageList
      });
    },
    //图片预览
    previewImage(e) {
      const index = e.currentTarget.dataset.index;
      const imageList = this.data.imageList;
      wx.previewImage({
        current: imageList[index], // 当前显示图片的http链接
        urls: imageList // 需要预览的图片http链接列表
      });
    },
    // 确认按钮事件
    onConfirm() {
      let params = {
        content: this.data.replyVal,
        tpCommentId: this.data.tpCommentId,
        trainingProjectId: this.properties.newsId
      }
      post(url.setcommentRaplyApi, params).then(res => {
        wx.showToast({
          title: '回复评论成功',
          icon: 'success',
          duration: 2000
        })
        this.getCommentList();
        this.hideModalFunc();
      })
    },
    setComment: function () {
      let params = {
        content: this.data.commentVal,
        bizType: this.properties.bizType == 2 ? "" : "2",
        imageUrls: JSON.stringify(this.data.uploadImgs),
        trainingProjectId: this.properties.newsId
      }
      post(url.setcommentApi, params).then(res => {
        wx.showToast({
          title: '评论成功',
          icon: 'success',
          duration: 2000,
          success: () => {
            this.setData({
              commentVal: "",
              imageList: []
            })
            this.getCommentList();
          }
        })

      })
    },
    // 取消按钮事件
    onCancel() {
      // 处理取消逻辑
      this.hideModalFunc();
    },
    // 显示模态框
    showModalFunc() {
      this.setData({
        showModal: true
      });
    },

    // 隐藏模态框
    hideModalFunc() {
      this.setData({
        showModal: false
      });
    },
  }
})