const TOKEN_KEY = 'auth_token';
const TOKEN_EXPIRY = 7 * 24 * 60 * 60 * 1000; // 7天有效期
const CryptoJS = require('./crypto-js'); // 需要引入加密库

Page({
  data: {
    account: '',
    password: '',
    captcha: '',
    captchaKey:'',
    isCounting: false,  // 验证码倒计时状态
    countdown: 60,      // 倒计时秒数
    isLoading: false    // 登录按钮加载状态
  },

  // 生命周期函数 - 检查本地token
  onLoad() {
    const tokenData = wx.getStorageSync(TOKEN_KEY);
    this.setData({ captchaKey: Math.floor(Math.random() * 999999)
      .toString()
      .padStart(6, "0")});
    if (tokenData && Date.now() < tokenData.expiry) {
      this.redirectToHome();
    }
  },

  // 输入事件处理
  onAccountInput(e) {
    this.setData({ account: e.detail.value.trim() });
  },
  onPasswordInput(e) {
    this.setData({ password: e.detail.value.trim() });
  },
  onCaptchaInput(e) {
    this.setData({ captcha: e.detail.value.trim() });
  },

  // 获取验证码逻辑
  async getCaptcha() {
    if (this.data.isCounting) return;
    this.startCountdown();
    try {
      const res = await wx.request({
        url: 'https://hqjddj.shanghaiairport.com/system/api//public/getCaptcha?companyCode=zhdj&siteCode=edu&captchaKey='+captchaKey,
        method: 'GET',
      });
      
      if (res.data) {
        wx.showToast({ title: '验证码已发送' });
        console.log(res.data,'验证码')
      } else {
        wx.showToast({ title: res.data.msg || '发送失败', icon: 'none' });
      }
    } catch (error) {
      wx.showToast({ title: '请求失败', icon: 'none' });
    }
  },

  // 验证码倒计时控制
  startCountdown() {
    this.setData({ isCounting: true });
    
    const timer = setInterval(() => {
      if (this.data.countdown <= 1) {
        clearInterval(timer);
        this.setData({ 
          isCounting: false,
          countdown: 60
        });
        return;
      }
      this.setData({ countdown: this.data.countdown - 1 });
    }, 1000);
  },

  // 登录处理
  async handleLogin() {
    if (this.data.isLoading) return;
    this.setData({ isLoading: true });
    
    try {
      const res = await wx.request({
        url: 'https://hqjddj.shanghaiairport.com/system/api/loginApp?companyCode=zhdj&siteCode=edu',
        method: 'POST',
        data: {
          account: this.data.account,
          password: this.encryptPassword(),
          captcha: this.data.captcha,
          captchaKey: this.data.captchaKey,
          captchaVal: this.data.captchaVal,
        }
      });

      this.setData({ isLoading: false });
      
      if (res.data.code === 200) {
        this.saveToken(res.data.data.token);
        this.redirectToHome();
      } else {
        wx.showToast({ title: res.data.msg || '登录失败', icon: 'none' });
      }
    } catch (error) {
      this.setData({ isLoading: false });
      wx.showToast({ title: '网络异常', icon: 'none' });
    }
  },
  // AES加密密码（需要配置加密密钥）
  encryptPassword() {
    const key = CryptoJS.enc.Utf8.parse('16位加密密钥');
    const encrypted = CryptoJS.AES.encrypt(
      this.data.password,
      key,
      { mode: CryptoJS.mode.ECB }
    );
    return encrypted.toString();
  },

  // Token存储
  saveToken(token) {
    wx.setStorageSync(TOKEN_KEY, {
      token: token,
      expiry: Date.now() + TOKEN_EXPIRY
    });
  },

  // 跳转到首页
  redirectToHome() {
    wx.switchTab({
      url: '/pages/home/index'
    });
  }
});